@@ -10,7 +10,6 @@ import (
 	"strings"
 	"time"
 
-	"github.com/docker/docker/api/types"
 	"github.com/docker/docker/api/types/container"
 	"github.com/docker/docker/client"
 
@@ -42,12 +41,12 @@ func NewCtl(ctx context.Context, host, pbmContainer string) (*Ctl, error) {
 }
 
 func (c *Ctl) PITRon() error {
-	out, err := c.RunCmd("pbm", "config", "--set", "pitr.enabled=true")
+	out, err := c.RunCmd("pbm", "config", "--set", "pitr.enabled=true", "--wait")
 	if err != nil {
 		return errors.Wrap(err, "config set pitr.enabled=true")
 	}
 
-	_, err = c.RunCmd("pbm", "config", "--set", "pitr.oplogSpanMin=1")
+	_, err = c.RunCmd("pbm", "config", "--set", "pitr.oplogSpanMin=1", "--wait")
 	if err != nil {
 		return errors.Wrap(err, "config set pitr.oplogSpanMin=1")
 	}
@@ -57,7 +56,7 @@ func (c *Ctl) PITRon() error {
 }
 
 func (c *Ctl) PITRoff() error {
-	out, err := c.RunCmd("pbm", "config", "--set", "pitr.enabled=false")
+	out, err := c.RunCmd("pbm", "config", "--set", "pitr.enabled=false", "--wait")
 	if err != nil {
 		return err
 	}
@@ -67,7 +66,7 @@ func (c *Ctl) PITRoff() error {
 }
 
 func (c *Ctl) ApplyConfig(file string) error {
-	out, err := c.RunCmd("pbm", "config", "--file", file)
+	out, err := c.RunCmd("pbm", "config", "--file", file, "--wait")
 	if err != nil {
 		return err
 	}
@@ -77,7 +76,7 @@ func (c *Ctl) ApplyConfig(file string) error {
 }
 
 func (c *Ctl) Resync() error {
-	out, err := c.RunCmd("pbm", "config", "--force-resync")
+	out, err := c.RunCmd("pbm", "config", "--force-resync", "--wait")
 	if err != nil {
 		return err
 	}
@@ -150,12 +149,12 @@ func skipCtl(str string) []byte {
 
 func (c *Ctl) CheckRestore(bcpName string, waitFor time.Duration) error {
 	type rlist struct {
-		Start    int
-		Status   defs.Status
-		Type     string
-		Name     string
-		Snapshot string
-		Error    string
+		Start    int         `json:"Start"`
+		Status   defs.Status `json:"Status"`
+		Type     string      `json:"Type"`
+		Name     string      `json:"Name"`
+		Snapshot string      `json:"Snapshot"`
+		Error    string      `json:"Error"`
 	}
 	tmr := time.NewTimer(waitFor)
 	tkr := time.NewTicker(500 * time.Millisecond)
@@ -287,7 +286,7 @@ func (c *Ctl) PITRestoreClusterTime(t, i uint32) error {
 }
 
 func (c *Ctl) RunCmd(cmds ...string) (string, error) {
-	execConf := types.ExecConfig{
+	execConf := container.ExecOptions{
 		Env:          c.env,
 		Cmd:          cmds,
 		AttachStderr: true,
@@ -298,7 +297,7 @@ func (c *Ctl) RunCmd(cmds ...string) (string, error) {
 		return "", errors.Wrap(err, "ContainerExecCreate")
 	}
 
-	container, err := c.cn.ContainerExecAttach(c.ctx, id.ID, types.ExecStartCheck{})
+	container, err := c.cn.ContainerExecAttach(c.ctx, id.ID, container.ExecAttachOptions{})
 	if err != nil {
 		return "", errors.Wrap(err, "attach to failed container")
 	}
