@@ -11,6 +11,7 @@ import (
 	"go.mongodb.org/mongo-driver/bson"
 	"go.mongodb.org/mongo-driver/bson/primitive"
 	"go.mongodb.org/mongo-driver/mongo"
+	"go.mongodb.org/mongo-driver/mongo/options"
 	"golang.org/x/sync/errgroup"
 
 	"github.com/percona/percona-backup-mongodb/pbm/archive"
@@ -41,19 +42,7 @@ func (b *Backup) doLogical(
 			return errors.Wrap(err, "check for timeseries")
 		}
 	}
-
-	var db, coll string
-	if util.IsSelective(bcp.Namespaces) {
-		// for selective backup, configsvr does not hold any data.
-		// only some collections from config db is required to restore cluster state
-		if inf.IsConfigSrv() {
-			db = "config"
-		} else {
-			db, coll = util.ParseNS(bcp.Namespaces[0])
-		}
-	}
-
-	nssSize, err := getNamespacesSize(ctx, b.nodeConn, db, coll)
+	nssSize, err := getNamespacesSize(ctx, b.nodeConn, bcp.Namespaces)
 	if err != nil {
 		return errors.Wrap(err, "get namespaces size")
 	}
@@ -102,7 +91,23 @@ func (b *Backup) doLogical(
 		rsMeta.FirstWriteTS,
 		func(ctx context.Context, w io.WriterTo, from, till primitive.Timestamp) (int64, error) {
 			filename := rsMeta.OplogName + "/" + FormatChunkName(from, till, bcp.Compression)
-			return storage.Upload(ctx, w, stg, bcp.Compression, bcp.CompressionLevel, filename, -1)
+
+			bytesPerSecond, err := getOplogBytesPerSecond(ctx, b.nodeConn)
+			if err != nil {
+				return 0, errors.Wrap(err, "oplog stats unavailable")
+			}
+
+			// Estimate size with headroom and enforce minimum part size for cloud uploads.
+			estimatedSize := bytesPerSecond * float64(till.T-from.T)
+			if bcp.Compression == compress.CompressionTypeNone {
+				estimatedSize *= 2
+			}
+			estimatedSize *= 2
+			if estimatedSize < 1<<30 { // 1 GiB
+				estimatedSize = 1 << 30
+			}
+
+			return storage.Upload(ctx, w, stg, bcp.Compression, bcp.CompressionLevel, filename, int64(estimatedSize))
 		})
 	// ensure slicer is stopped in any case (done, error or canceled)
 	defer stopOplogSlicer() //nolint:errcheck
@@ -133,61 +138,69 @@ func (b *Backup) doLogical(
 		}
 	}
 
-	var dump io.WriterTo
-	if len(nssSize) == 0 {
-		dump = snapshot.DummyBackup{}
-	} else {
-		numParallelColls := b.numParallelColls
-		if bcp.NumParallelColls != nil {
-			if *bcp.NumParallelColls > 0 {
-				numParallelColls = int(*bcp.NumParallelColls)
-			} else {
-				l.Warning("invalid value of NumParallelCollections (%v). fallback to %v",
-					numParallelColls, b.numParallelColls)
-			}
-		}
-
-		l.Debug("dumping up to %d collections in parallel", numParallelColls)
-
-		dump, err = snapshot.NewBackup(b.brief.URI, numParallelColls, db, coll)
-		if err != nil {
-			return errors.Wrap(err, "init mongodump options")
+	numParallelColls := b.numParallelColls
+	if bcp.NumParallelColls != nil {
+		if *bcp.NumParallelColls > 0 {
+			numParallelColls = int(*bcp.NumParallelColls)
+		} else {
+			l.Warning("invalid value of NumParallelCollections (%v). fallback to %v",
+				numParallelColls, b.numParallelColls)
 		}
 	}
+	l.Debug("dumping up to %d collections in parallel", numParallelColls)
 
 	nsFilter := archive.DefaultNSFilter
 	docFilter := archive.DefaultDocFilter
-	if inf.IsConfigSrv() && util.IsSelective(bcp.Namespaces) {
-		chunkSelector, err := createBackupChunkSelector(ctx, b.leadConn, bcp.Namespaces)
-		if err != nil {
-			return errors.Wrap(err, "fetch uuids")
-		}
+	if util.IsSelective(bcp.Namespaces) {
+		if inf.IsConfigSrv() {
+			chunkSelector, err := createBackupChunkSelector(ctx, b.leadConn, bcp.Namespaces)
+			if err != nil {
+				return errors.Wrap(err, "fetch uuids")
+			}
 
-		nsFilter = makeConfigsvrNSFilter()
-		docFilter = makeConfigsvrDocFilter(bcp.Namespaces, chunkSelector)
+			docFilter = makeConfigsvrDocFilter(bcp.Namespaces, chunkSelector)
+			nsFilter = makeConfigsvrNSFilter(bcp.Namespaces)
+
+		} else {
+			nsFilter = util.MakeSelectedPred(bcp.Namespaces)
+		}
 	}
 
 	snapshotSize, err := snapshot.UploadDump(ctx,
-		dump,
+		func(newFile archive.NewWriter) error {
+			bcp, err := archive.NewBackup(ctx, archive.BackupOptions{
+				Client:        b.nodeConn,
+				NewFile:       newFile,
+				NSFilter:      nsFilter,
+				DocFilter:     docFilter,
+				ParallelColls: numParallelColls,
+			})
+			if err != nil {
+				return errors.Wrap(err, "new backup")
+			}
+
+			return bcp.Run(ctx)
+		},
 		func(ns, ext string, r io.Reader) error {
-			stg, err := util.StorageFromConfig(&b.config.Storage, l)
+			stg, err := util.StorageFromConfig(&b.config.Storage, b.brief.Me, l)
 			if err != nil {
 				return errors.Wrap(err, "get storage")
 			}
-
 			filepath := path.Join(bcp.Name, rsMeta.Name, ns+ext)
-			return stg.Save(filepath, r, nssSize[ns])
+			return stg.Save(filepath, r, storage.Size(nssSize[ns]))
 		},
-		snapshot.UploadDumpOptions{
-			Compression:      bcp.Compression,
-			CompressionLevel: bcp.CompressionLevel,
-			NSFilter:         nsFilter,
-			DocFilter:        docFilter,
-		})
+		bcp.Compression,
+		bcp.CompressionLevel)
 	if err != nil {
-		return errors.Wrap(err, "mongodump")
+		return errors.Wrap(err, "dump")
 	}
-	l.Info("mongodump finished, waiting for the oplog")
+
+	err = archive.GenerateV1FromV2(ctx, stg, bcp.Name, rsMeta.Name)
+	if err != nil {
+		return errors.Wrap(err, "generate archive meta v1")
+	}
+
+	l.Info("dump finished, waiting for the oplog")
 
 	err = ChangeRSState(b.leadConn, bcp.Name, rsMeta.Name, defs.StatusDumpDone, "")
 	if err != nil {
@@ -223,7 +236,7 @@ func (b *Backup) doLogical(
 		}
 	}
 
-	err = IncBackupSize(ctx, b.leadConn, bcp.Name, snapshotSize+oplogSize)
+	err = IncBackupSize(ctx, b.leadConn, bcp.Name, snapshotSize+oplogSize, nil)
 	if err != nil {
 		return errors.Wrap(err, "inc backup size")
 	}
@@ -351,17 +364,16 @@ func createBackupChunkSelector(ctx context.Context, m connect.Client, nss []stri
 	return chunkSelector, nil
 }
 
-func makeConfigsvrNSFilter() archive.NSFilterFn {
+func makeConfigsvrNSFilter(nss []string) archive.NSFilterFn {
+	selected := make([]string, len(nss)+len(archive.RouterConfigCollections))
+	n := copy(selected, nss)
+
 	// list of required namespaces for further selective restore
-	allowed := map[string]bool{
-		"config.databases":   true,
-		"config.collections": true,
-		"config.chunks":      true,
+	for i, name := range archive.RouterConfigCollections {
+		selected[n+i] = "config." + name
 	}
 
-	return func(ns string) bool {
-		return allowed[ns]
-	}
+	return util.MakeSelectedPred(selected)
 }
 
 func makeConfigsvrDocFilter(nss []string, selector util.ChunkSelector) archive.DocFilterFn {
@@ -384,18 +396,19 @@ func makeConfigsvrDocFilter(nss []string, selector util.ChunkSelector) archive.D
 			return selector.Selected(doc)
 		}
 
+		// Config Shard should keep non-config collections
+		if selectedNS(ns) {
+			return true
+		}
+
 		return false
 	}
 }
 
-func getNamespacesSize(ctx context.Context, m *mongo.Client, db, coll string) (map[string]int64, error) {
+func getNamespacesSize(ctx context.Context, m *mongo.Client, nss []string) (map[string]int64, error) {
 	rv := make(map[string]int64)
 
-	q := bson.D{}
-	if db != "" {
-		q = append(q, bson.E{"name", db})
-	}
-	dbs, err := m.ListDatabaseNames(ctx, q)
+	dbs, err := m.ListDatabaseNames(ctx, bson.D{})
 	if err != nil {
 		return nil, errors.Wrap(err, "list databases")
 	}
@@ -403,6 +416,8 @@ func getNamespacesSize(ctx context.Context, m *mongo.Client, db, coll string) (m
 		return rv, nil
 	}
 
+	isSelected := util.MakeSelectedPred(nss)
+
 	mu := sync.Mutex{}
 	eg, ctx := errgroup.WithContext(ctx)
 
@@ -410,11 +425,7 @@ func getNamespacesSize(ctx context.Context, m *mongo.Client, db, coll string) (m
 		db := db
 
 		eg.Go(func() error {
-			q := bson.D{}
-			if coll != "" {
-				q = append(q, bson.E{"name", coll})
-			}
-			res, err := m.Database(db).ListCollectionSpecifications(ctx, q)
+			res, err := m.Database(db).ListCollectionSpecifications(ctx, bson.D{})
 			if err != nil {
 				return errors.Wrapf(err, "list collections for %q", db)
 			}
@@ -425,14 +436,16 @@ func getNamespacesSize(ctx context.Context, m *mongo.Client, db, coll string) (m
 			eg, ctx := errgroup.WithContext(ctx)
 
 			for _, coll := range res {
-				coll := coll
-
 				if coll.Type == "view" || strings.HasPrefix(coll.Name, "system.buckets.") {
 					continue
 				}
 
+				ns := db + "." + coll.Name
+				if !isSelected(ns) {
+					continue
+				}
+
 				eg.Go(func() error {
-					ns := db + "." + coll.Name
 					res := m.Database(db).RunCommand(ctx, bson.D{{"collStats", coll.Name}})
 					if err := res.Err(); err != nil {
 						return errors.Wrapf(err, "collStats %q", ns)
@@ -462,6 +475,58 @@ func getNamespacesSize(ctx context.Context, m *mongo.Client, db, coll string) (m
 	return rv, err
 }
 
+// getOplogBytesPerSecond returns the average number of bytes written to the
+// replica‑set oplog each second. It fetches the total size of the
+// `local.oplog.rs` collection, reads the timestamps of the oldest and newest
+// entries, and divides the size by the seconds between those timestamps.
+func getOplogBytesPerSecond(ctx context.Context, m *mongo.Client) (float64, error) {
+	coll := m.Database("local").Collection("oplog.rs")
+
+	var stat struct {
+		Size int64 `bson:"size"`
+	}
+	cmd := bson.D{{Key: "collStats", Value: "oplog.rs"}}
+	res := m.Database("local").RunCommand(ctx, cmd)
+	if res.Err() != nil {
+		return 0, errors.Wrap(res.Err(), "collStats local.oplog.rs")
+	}
+	if err := res.Decode(&stat); err != nil {
+		return 0, errors.Wrap(err, "decode collStats")
+	}
+
+	oldestTS, err := oplogTimestamp(ctx, coll, 1)
+	if err != nil {
+		return 0, err
+	}
+	newestTS, err := oplogTimestamp(ctx, coll, -1)
+	if err != nil {
+		return 0, err
+	}
+
+	duration := int64(newestTS.T - oldestTS.T)
+	if duration <= 0 {
+		return 0, errors.New("invalid oplog window duration")
+	}
+
+	return float64(stat.Size) / float64(duration), nil
+}
+
+// oplogTimestamp returns the timestamp of the first (sort=1) or last (sort=-1) document in the oplog.
+func oplogTimestamp(ctx context.Context, coll *mongo.Collection, sort int) (primitive.Timestamp, error) {
+	var doc bson.M
+	err := coll.FindOne(ctx, bson.D{}, options.FindOne().SetSort(bson.D{{Key: "$natural", Value: sort}})).Decode(&doc)
+	if err != nil {
+		return primitive.Timestamp{}, errors.Wrap(err, "query oplog timestamp")
+	}
+
+	ts, ok := doc["ts"].(primitive.Timestamp)
+	if !ok {
+		return primitive.Timestamp{}, errors.New("missing or invalid 'ts' field")
+	}
+
+	return ts, nil
+}
+
 func (b *Backup) checkForTimeseries(ctx context.Context, nss []string) error {
 	if !b.brief.Version.IsShardedTimeseriesSupported() || !b.brief.Sharded {
 		return nil
