@@ -25,7 +25,7 @@ type dbMangerImpl struct {
 	conn connect.Client
 }
 
-func NewDBManager(conn connect.Client) Manager {
+func NewDBManager(conn connect.Client) *dbMangerImpl {
 	return &dbMangerImpl{conn: conn}
 }
 
@@ -81,15 +81,34 @@ func getBackupMeta(ctx context.Context, conn connect.Client, clause bson.D) (*Ba
 }
 
 func ChangeBackupStateOPID(conn connect.Client, opid string, s defs.Status, msg string) error {
-	return changeBackupState(context.Background(), conn, bson.D{{"opid", opid}}, s, msg)
+	return changeBackupState(context.TODO(),
+		conn, bson.D{{"opid", opid}}, time.Now().UTC().Unix(), s, msg)
 }
 
 func ChangeBackupState(conn connect.Client, bcpName string, s defs.Status, msg string) error {
-	return changeBackupState(context.Background(), conn, bson.D{{"name", bcpName}}, s, msg)
+	return changeBackupState(context.TODO(),
+		conn, bson.D{{"name", bcpName}}, time.Now().UTC().Unix(), s, msg)
 }
 
-func changeBackupState(ctx context.Context, conn connect.Client, clause bson.D, s defs.Status, msg string) error {
-	ts := time.Now().UTC().Unix()
+func ChangeBackupStateWithUnixTime(
+	ctx context.Context,
+	conn connect.Client,
+	bcpName string,
+	s defs.Status,
+	unix int64,
+	msg string,
+) error {
+	return changeBackupState(ctx, conn, bson.D{{"name", bcpName}}, time.Now().UTC().Unix(), s, msg)
+}
+
+func changeBackupState(
+	ctx context.Context,
+	conn connect.Client,
+	clause bson.D,
+	ts int64,
+	s defs.Status,
+	msg string,
+) error {
 	_, err := conn.BcpCollection().UpdateOne(
 		ctx,
 		clause,
@@ -200,14 +219,52 @@ func ChangeRSState(conn connect.Client, bcpName, rsName string, s defs.Status, m
 	return err
 }
 
-func IncBackupSize(ctx context.Context, conn connect.Client, bcpName string, size int64) error {
+// IncBackupSize increments total backup size.
+func IncBackupSize(
+	ctx context.Context,
+	conn connect.Client,
+	bcpName string,
+	size int64,
+	sizeUncompressed *int64,
+) error {
+	update := bson.D{
+		{"$inc", bson.M{"size": size}},
+	}
+	if sizeUncompressed != nil {
+		update = append(
+			update,
+			bson.E{"$inc", bson.M{"size_uncompressed": sizeUncompressed}},
+		)
+	}
+
 	_, err := conn.BcpCollection().UpdateOne(ctx,
 		bson.D{{"name", bcpName}},
-		bson.D{{"$inc", bson.M{"size": size}}})
+		update,
+	)
 
 	return err
 }
 
+// SetBackupSizeForRS sets size of backup for specified RS.
+func SetBackupSizeForRS(
+	ctx context.Context,
+	conn connect.Client,
+	bcpName,
+	rsName string,
+	size int64,
+	sizeUncompressed int64,
+) error {
+	_, err := conn.BcpCollection().UpdateOne(
+		ctx,
+		bson.D{{"name", bcpName}, {"replsets.name", rsName}},
+		bson.D{
+			{"$set", bson.M{"replsets.$.size": size}},
+			{"$set", bson.M{"replsets.$.size_uncompressed": sizeUncompressed}},
+		},
+	)
+	return err
+}
+
 func SetRSLastWrite(conn connect.Client, bcpName, rsName string, ts primitive.Timestamp) error {
 	_, err := conn.BcpCollection().UpdateOne(
 		context.Background(),
@@ -228,13 +285,19 @@ func LastIncrementalBackup(ctx context.Context, conn connect.Client) (*BackupMet
 // or nil if there is no such backup yet. If ts isn't nil it will
 // search for the most recent backup that finished before specified timestamp
 func GetLastBackup(ctx context.Context, conn connect.Client, before *primitive.Timestamp) (*BackupMeta, error) {
-	return getRecentBackup(ctx, conn, nil, before, -1,
-		bson.D{{"nss", nil}, {"type", bson.M{"$ne": defs.ExternalBackup}}})
+	return getRecentBackup(ctx, conn, nil, before, -1, bson.D{
+		{"nss", nil},
+		{"type", bson.M{"$ne": defs.ExternalBackup}},
+		{"store.profile", nil},
+	})
 }
 
 func GetFirstBackup(ctx context.Context, conn connect.Client, after *primitive.Timestamp) (*BackupMeta, error) {
-	return getRecentBackup(ctx, conn, after, nil, 1,
-		bson.D{{"nss", nil}, {"type", bson.M{"$ne": defs.ExternalBackup}}})
+	return getRecentBackup(ctx, conn, after, nil, 1, bson.D{
+		{"nss", nil},
+		{"type", bson.M{"$ne": defs.ExternalBackup}},
+		{"store.profile", nil},
+	})
 }
 
 func getRecentBackup(
@@ -273,19 +336,19 @@ func getRecentBackup(
 
 func FindBaseSnapshotLWAfter(
 	ctx context.Context,
-	cc connect.Client,
+	conn connect.Client,
 	lw primitive.Timestamp,
 ) (primitive.Timestamp, error) {
-	return findBaseSnapshotLWImpl(ctx, cc, bson.M{"$gt": lw}, 1)
+	return findBaseSnapshotLWImpl(ctx, conn, bson.M{"$gt": lw}, 1)
 }
 
 func FindBaseSnapshotLWBefore(
 	ctx context.Context,
-	cc connect.Client,
+	conn connect.Client,
 	lw primitive.Timestamp,
 	exclude primitive.Timestamp,
 ) (primitive.Timestamp, error) {
-	return findBaseSnapshotLWImpl(ctx, cc, bson.M{"$lt": lw, "$ne": exclude}, -1)
+	return findBaseSnapshotLWImpl(ctx, conn, bson.M{"$lt": lw, "$ne": exclude}, -1)
 }
 
 func findBaseSnapshotLWImpl(
@@ -297,6 +360,7 @@ func findBaseSnapshotLWImpl(
 	f := bson.D{
 		{"nss", nil},
 		{"type", bson.M{"$ne": defs.ExternalBackup}},
+		{"store.profile", nil},
 		{"last_write_ts", lwCond},
 		{"status", defs.StatusDone},
 	}
