@@ -17,6 +17,7 @@ import (
 	"go.mongodb.org/mongo-driver/bson"
 	"go.mongodb.org/mongo-driver/bson/primitive"
 	"go.mongodb.org/mongo-driver/mongo"
+	"go.mongodb.org/mongo-driver/mongo/writeconcern"
 
 	"github.com/percona/percona-backup-mongodb/pbm/archive"
 	"github.com/percona/percona-backup-mongodb/pbm/backup"
@@ -48,7 +49,8 @@ type Restore struct {
 	bcpStg   storage.Storage
 	oplogStg storage.Storage
 
-	numParallelColls int
+	numParallelColls          int
+	numInsertionWorkersPerCol int
 	// Shards to participate in restore. Num of shards in bcp could
 	// be less than in the cluster and this is ok. Only these shards
 	// would be expected to run restore (distributed transactions sync,
@@ -56,11 +58,11 @@ type Restore struct {
 	//
 	// Only the restore leader would have this info.
 	shards []topo.Shard
-	// rsMap is mapping between old and new replset names. used for data restore.
-	// empty if all replset names are the same
+	// rsMap is mapping between old and new replset names, used for data restore.
+	// It's empty if all replset names are the same.
 	rsMap map[string]string
-	// sMap is mapping between old and new shard names. used for router config update.
-	// empty if all shard names are the same
+	// sMap is mapping between old and new shard names, used for router config update.
+	// It's empty if all shard names are the same.
 	sMap map[string]string
 
 	log  log.LogEvent
@@ -75,6 +77,13 @@ type oplogRange struct {
 	storage storage.Storage
 }
 
+// configDatabasesDoc represents document in config.databases collection
+type configDatabasesDoc struct {
+	ID      string `bson:"_id"`
+	Primary string `bson:"primary"`
+	Version bson.D `bson:"version"`
+}
+
 // PBM restore from temp collections (pbmRUsers/pbmRRoles)should be used
 type restoreUsersAndRolesOption bool
 
@@ -85,7 +94,8 @@ func New(
 	brief topo.NodeBrief,
 	cfg *config.Config,
 	rsMap map[string]string,
-	numParallelColls int,
+	numParallelColls,
+	numInsertionWorkersPerCol int,
 ) *Restore {
 	if rsMap == nil {
 		rsMap = make(map[string]string)
@@ -99,9 +109,9 @@ func New(
 
 		cfg: cfg,
 
-		numParallelColls: numParallelColls,
-
-		indexCatalog: idx.NewIndexCatalog(),
+		numParallelColls:          numParallelColls,
+		numInsertionWorkersPerCol: numInsertionWorkersPerCol,
+		indexCatalog:              idx.NewIndexCatalog(),
 	}
 }
 
@@ -126,8 +136,11 @@ func (r *Restore) exit(ctx context.Context, err error) {
 }
 
 // resolveNamespace resolves final namespace(s) based on the backup namespace,
-// restore namespace, and option whether we should restore users&roles
-func resolveNamespace(nssBackup, nssRestore []string, usingUsersAndRoles bool) []string {
+// restore namespace, cloning options and option whether we should restore users&roles
+func resolveNamespace(nssBackup, nssRestore []string, cloneNS snapshot.CloneNS, usingUsersAndRoles bool) []string {
+	if cloneNS.IsSpecified() {
+		return []string{cloneNS.FromNS}
+	}
 	if util.IsSelective(nssRestore) {
 		if usingUsersAndRoles {
 			var nss []string
@@ -149,7 +162,14 @@ func resolveNamespace(nssBackup, nssRestore []string, usingUsersAndRoles bool) [
 }
 
 // shouldRestoreUsersAndRoles determines whether user&roles should be restored from the backup
-func shouldRestoreUsersAndRoles(nssBackup, nssRestore []string, usingUsersAndRoles bool) restoreUsersAndRolesOption {
+func shouldRestoreUsersAndRoles(
+	nssBackup, nssRestore []string,
+	cloneNS snapshot.CloneNS,
+	usingUsersAndRoles bool,
+) restoreUsersAndRolesOption {
+	if cloneNS.IsSpecified() {
+		return false
+	}
 	if util.IsSelective(nssBackup) {
 		return false
 	}
@@ -178,13 +198,26 @@ func (r *Restore) Snapshot(
 		return err
 	}
 
-	r.bcpStg, err = util.StorageFromConfig(&bcp.Store.StorageConf, r.log)
+	r.bcpStg, err = util.StorageFromConfig(&bcp.Store.StorageConf, r.brief.Me, r.log)
 	if err != nil {
 		return errors.Wrap(err, "get backup storage")
 	}
 
-	nss := resolveNamespace(bcp.Namespaces, cmd.Namespaces, cmd.UsersAndRoles)
-	usersAndRolesOpt := shouldRestoreUsersAndRoles(bcp.Namespaces, cmd.Namespaces, cmd.UsersAndRoles)
+	cloneNS := snapshot.CloneNS{FromNS: cmd.NamespaceFrom, ToNS: cmd.NamespaceTo}
+	if r.brief.Sharded && cloneNS.IsSpecified() {
+		return errors.New("Namespace cloning is not supported in sharded cluster")
+	}
+
+	nss := resolveNamespace(
+		bcp.Namespaces,
+		cmd.Namespaces,
+		cloneNS,
+		cmd.UsersAndRoles)
+	usersAndRolesOpt := shouldRestoreUsersAndRoles(
+		bcp.Namespaces,
+		cmd.Namespaces,
+		cloneNS,
+		cmd.UsersAndRoles)
 
 	err = setRestoreBackup(ctx, r.leadConn, r.name, cmd.BackupName, nss)
 	if err != nil {
@@ -210,12 +243,30 @@ func (r *Restore) Snapshot(
 		return err
 	}
 
-	err = r.toState(ctx, defs.StatusRunning, &defs.WaitActionStart)
+	err = r.checkForCompatibility(ctx, util.MakeReverseRSMapFunc(r.rsMap)(r.brief.SetName), bcp)
 	if err != nil {
 		return err
 	}
 
-	err = r.RunSnapshot(ctx, dump, bcp, nss, usersAndRolesOpt)
+	err = r.toState(ctx, defs.StatusCleanupCluster, &defs.WaitActionStart)
+	if err != nil {
+		return err
+	}
+
+	// drop sharded dbs on sharded cluster, on each shard (not CSRS), only for full restore
+	if r.nodeInfo.IsSharded() && !r.nodeInfo.IsConfigSrv() && !util.IsSelective(nss) {
+		err = r.dropShardedDBs(ctx, bcp)
+		if err != nil {
+			return err
+		}
+	}
+
+	err = r.toState(ctx, defs.StatusRunning, nil)
+	if err != nil {
+		return err
+	}
+
+	err = r.RunSnapshot(ctx, dump, bcp, nss, cloneNS, usersAndRolesOpt)
 	if err != nil {
 		return err
 	}
@@ -228,17 +279,26 @@ func (r *Restore) Snapshot(
 	oplogRanges := []oplogRange{
 		{chunks: chunks, storage: r.bcpStg},
 	}
-	oplogOption := &applyOplogOption{end: &bcp.LastWriteTS, nss: nss}
+	oplogOption := &applyOplogOption{
+		end:     &bcp.LastWriteTS,
+		nss:     nss,
+		cloudNS: cloneNS,
+	}
 	if r.nodeInfo.IsConfigSrv() && util.IsSelective(nss) {
 		oplogOption.nss = []string{"config.databases"}
 		oplogOption.filter = newConfigsvrOpFilter(nss)
 	}
+
 	err = r.applyOplog(ctx, oplogRanges, oplogOption)
 	if err != nil {
 		return err
 	}
 
-	err = r.restoreIndexes(ctx, oplogOption.nss)
+	if cloneNS.IsSpecified() {
+		err = r.restoreIndexes(ctx, []string{cloneNS.ToNS})
+	} else {
+		err = r.restoreIndexes(ctx, oplogOption.nss)
+	}
 	if err != nil {
 		return errors.Wrap(err, "restore indexes")
 	}
@@ -255,6 +315,9 @@ func newConfigsvrOpFilter(nss []string) oplog.OpFilter {
 	selected := util.MakeSelectedPred(nss)
 
 	return func(r *oplog.Record) bool {
+		if selected(r.Namespace) {
+			return true
+		}
 		if r.Namespace != "config.databases" {
 			return false
 		}
@@ -296,17 +359,30 @@ func (r *Restore) PITR(
 			"Try to set an earlier snapshot. Or leave the snapshot empty so PBM will choose one.")
 	}
 
-	r.bcpStg, err = util.StorageFromConfig(&bcp.Store.StorageConf, r.log)
+	r.bcpStg, err = util.StorageFromConfig(&bcp.Store.StorageConf, r.brief.Me, r.log)
 	if err != nil {
 		return errors.Wrap(err, "get backup storage")
 	}
-	r.oplogStg, err = util.GetStorage(ctx, r.leadConn, log.LogEventFromContext(ctx))
+	r.oplogStg, err = util.GetStorage(ctx, r.leadConn, r.nodeInfo.Me, log.LogEventFromContext(ctx))
 	if err != nil {
 		return errors.Wrap(err, "get oplog storage")
 	}
 
-	nss := resolveNamespace(bcp.Namespaces, cmd.Namespaces, cmd.UsersAndRoles)
-	usersAndRolesOpt := shouldRestoreUsersAndRoles(bcp.Namespaces, cmd.Namespaces, cmd.UsersAndRoles)
+	cloneNS := snapshot.CloneNS{FromNS: cmd.NamespaceFrom, ToNS: cmd.NamespaceTo}
+	if r.brief.Sharded && cloneNS.IsSpecified() {
+		return errors.New("Namespace cloning is not supported in sharded cluster")
+	}
+
+	nss := resolveNamespace(
+		bcp.Namespaces,
+		cmd.Namespaces,
+		cloneNS,
+		cmd.UsersAndRoles)
+	usersAndRolesOpt := shouldRestoreUsersAndRoles(
+		bcp.Namespaces,
+		cmd.Namespaces,
+		cloneNS,
+		cmd.UsersAndRoles)
 
 	if r.nodeInfo.IsLeader() {
 		err = SetOplogTimestamps(ctx, r.leadConn, r.name, 0, int64(cmd.OplogTS.T))
@@ -353,12 +429,30 @@ func (r *Restore) PITR(
 		return err
 	}
 
-	err = r.toState(ctx, defs.StatusRunning, &defs.WaitActionStart)
+	err = r.checkForCompatibility(ctx, util.MakeReverseRSMapFunc(r.rsMap)(r.brief.SetName), bcp)
 	if err != nil {
 		return err
 	}
 
-	err = r.RunSnapshot(ctx, dump, bcp, nss, usersAndRolesOpt)
+	err = r.toState(ctx, defs.StatusCleanupCluster, &defs.WaitActionStart)
+	if err != nil {
+		return err
+	}
+
+	// drop sharded dbs on sharded cluster, on each shard (not CSRS), only for full restore
+	if r.nodeInfo.IsSharded() && !r.nodeInfo.IsConfigSrv() && !util.IsSelective(nss) {
+		err = r.dropShardedDBs(ctx, bcp)
+		if err != nil {
+			return err
+		}
+	}
+
+	err = r.toState(ctx, defs.StatusRunning, nil)
+	if err != nil {
+		return err
+	}
+
+	err = r.RunSnapshot(ctx, dump, bcp, nss, cloneNS, usersAndRolesOpt)
 	if err != nil {
 		return err
 	}
@@ -372,9 +466,14 @@ func (r *Restore) PITR(
 		{chunks: bcpChunks, storage: r.bcpStg},
 		{chunks: chunks, storage: r.oplogStg},
 	}
-	oplogOption := applyOplogOption{end: &cmd.OplogTS, nss: nss}
+	oplogOption := applyOplogOption{
+		end:     &cmd.OplogTS,
+		nss:     nss,
+		cloudNS: cloneNS,
+	}
 	if r.nodeInfo.IsConfigSrv() && util.IsSelective(nss) {
 		oplogOption.nss = []string{"config.databases"}
+		oplogOption.nss = append(oplogOption.nss, nss...)
 		oplogOption.filter = newConfigsvrOpFilter(nss)
 	}
 	err = r.applyOplog(ctx, oplogRanges, &oplogOption)
@@ -382,7 +481,11 @@ func (r *Restore) PITR(
 		return err
 	}
 
-	err = r.restoreIndexes(ctx, oplogOption.nss)
+	if cloneNS.IsSpecified() {
+		err = r.restoreIndexes(ctx, []string{cloneNS.ToNS})
+	} else {
+		err = r.restoreIndexes(ctx, oplogOption.nss)
+	}
 	if err != nil {
 		return errors.Wrap(err, "restore indexes")
 	}
@@ -432,7 +535,7 @@ func (r *Restore) ReplayOplog(ctx context.Context, cmd *ctrl.ReplayCmd, opid ctr
 		return r.Done(ctx) // skip. no oplog for current rs
 	}
 
-	r.oplogStg, err = util.GetStorage(ctx, r.leadConn, log.LogEventFromContext(ctx))
+	r.oplogStg, err = util.GetStorage(ctx, r.leadConn, r.nodeInfo.Me, log.LogEventFromContext(ctx))
 	if err != nil {
 		return errors.Wrapf(err, "get oplog storage")
 	}
@@ -578,6 +681,7 @@ func LookupBackupMeta(
 	ctx context.Context,
 	conn connect.Client,
 	backupName string,
+	node string,
 ) (*backup.BackupMeta, error) {
 	bcp, err := backup.NewDBManager(conn).GetBackupByName(ctx, backupName)
 	if err == nil {
@@ -588,7 +692,7 @@ func LookupBackupMeta(
 	}
 
 	var stg storage.Storage
-	stg, err = util.GetStorage(ctx, conn, log.LogEventFromContext(ctx))
+	stg, err = util.GetStorage(ctx, conn, node, log.LogEventFromContext(ctx))
 	if err != nil {
 		return nil, errors.Wrap(err, "get storage")
 	}
@@ -639,7 +743,10 @@ func (r *Restore) setShards(ctx context.Context, bcp *backup.BackupMeta) error {
 	return nil
 }
 
-var ErrNoDataForShard = errors.New("no data for shard")
+var (
+	ErrNoDataForShard     = errors.New("no data for shard")
+	ErrNoDataForConfigsvr = errors.New("no data for the config server or sole rs in backup")
+)
 
 func (r *Restore) snapshotObjects(bcp *backup.BackupMeta) (string, []oplog.OplogChunk, error) {
 	var ok bool
@@ -655,7 +762,7 @@ func (r *Restore) snapshotObjects(bcp *backup.BackupMeta) (string, []oplog.Oplog
 	}
 	if !ok {
 		if r.nodeInfo.IsLeader() {
-			return "", nil, errors.New("no data for the config server or sole rs in backup")
+			return "", nil, ErrNoDataForConfigsvr
 		}
 		return "", nil, ErrNoDataForShard
 	}
@@ -755,93 +862,158 @@ func (r *Restore) toState(ctx context.Context, status defs.Status, wait *time.Du
 	return toState(ctx, r.leadConn, status, r.name, r.nodeInfo, r.reconcileStatus, wait)
 }
 
+// dropShardedDBs drop all sharded databases present in the backup.
+// Backup is specified with bcp parameter.
+// For each sharded database present in the backup _shardsvrDropDatabase command
+// is used to drop the database from the config srv and all shards.
+func (r *Restore) dropShardedDBs(ctx context.Context, bcp *backup.BackupMeta) error {
+	dbsInBcp, err := r.getDBsFromBackup(bcp)
+	if err != nil {
+		return errors.Wrap(err, "get dbs from backup")
+	}
+
+	// make cluster-wide drop for each db from the backup
+	for _, db := range dbsInBcp {
+		var configDBDoc configDatabasesDoc
+		err := r.leadConn.ConfigDatabase().
+			Collection("databases").
+			FindOne(ctx, bson.D{{"_id", db}}).
+			Decode(&configDBDoc)
+		if err != nil {
+			if errors.Is(err, mongo.ErrNoDocuments) {
+				continue
+			}
+			return errors.Wrapf(err, "get config.databases for %q", db)
+		}
+
+		if configDBDoc.Primary != r.nodeInfo.SetName {
+			// this shard is not primary shard for this db, so ignore it
+			continue
+		}
+
+		cmd := bson.D{
+			{"_shardsvrDropDatabase", 1},
+			{"databaseVersion", configDBDoc.Version},
+			{"writeConcern", writeconcern.Majority()},
+		}
+		res := r.nodeConn.Database(db).RunCommand(ctx, cmd)
+		if err := res.Err(); err != nil {
+			return errors.Wrapf(err, "_shardsvrDropDatabase for %q", db)
+		}
+		r.log.Debug("drop %q", db)
+	}
+
+	return nil
+}
+
+// getDBsFromBackup returns all databases present in backup metadata file
+// for each replicaset.
+func (r *Restore) getDBsFromBackup(bcp *backup.BackupMeta) ([]string, error) {
+	rsName := util.MakeReverseRSMapFunc(r.rsMap)(r.brief.SetName)
+	filepath := path.Join(bcp.Name, rsName, archive.MetaFile)
+	rdr, err := r.bcpStg.SourceReader(filepath)
+	if err != nil {
+		return nil, errors.Wrap(err, "read metadata file")
+	}
+	defer rdr.Close()
+
+	meta, err := archive.ReadMetadata(rdr)
+	if err != nil {
+		return nil, errors.Wrap(err, "get metadata")
+	}
+
+	uniqueDbs := map[string]bool{}
+	for _, ns := range meta.Namespaces {
+		uniqueDbs[ns.Database] = true
+	}
+	dbs := []string{}
+	for db := range uniqueDbs {
+		dbs = append(dbs, db)
+	}
+
+	return dbs, nil
+}
+
 func (r *Restore) RunSnapshot(
 	ctx context.Context,
 	dump string,
 	bcp *backup.BackupMeta,
 	nss []string,
+	cloneNS snapshot.CloneNS,
 	usersAndRolesOpt restoreUsersAndRolesOption,
 ) error {
-	var rdr io.ReadCloser
-	var err error
 	if version.IsLegacyArchive(bcp.PBMVersion) {
-		sr, err := r.bcpStg.SourceReader(dump)
-		if err != nil {
-			return errors.Wrapf(err, "get object %s for the storage", dump)
-		}
-		defer sr.Close()
-
-		rdr, err = compress.Decompress(sr, bcp.Compression)
-		if err != nil {
-			return errors.Wrapf(err, "decompress object %s", dump)
-		}
-	} else {
-		if !util.IsSelective(nss) {
-			nss = bcp.Namespaces
-		}
-		if !util.IsSelective(nss) {
-			nss = []string{"*.*"}
+		if util.IsSelective(bcp.Namespaces) || util.IsSelective(nss) {
+			return errors.New("selective restore is not supported from legacy backup")
 		}
 
-		mapRS := util.MakeReverseRSMapFunc(r.rsMap)
-		if r.nodeInfo.IsConfigSrv() && util.IsSelective(nss) {
-			// restore cluster specific configs only
-			if err := r.configsvrRestore(ctx, bcp, nss, mapRS); err != nil {
-				return err
+		return r.restoreLegacyArchive(ctx, dump, bcp)
+	}
+
+	if !util.IsSelective(nss) {
+		nss = bcp.Namespaces
+	}
+	if !util.IsSelective(nss) {
+		nss = []string{"*.*"}
+	}
+
+	mapRS := util.MakeReverseRSMapFunc(r.rsMap)
+
+	r.log.Debug("restoring up to %d collections in parallel", r.numParallelColls)
+
+	rdr, err := snapshot.DownloadDump(
+		func(ns string) (io.ReadCloser, error) {
+			stg, err := util.StorageFromConfig(&bcp.Store.StorageConf, r.brief.Me, r.log)
+			if err != nil {
+				return nil, errors.Wrap(err, "get storage")
 			}
-			if !usersAndRolesOpt {
-				return nil
+			// while importing backup made by RS with another name
+			// that current RS we can't use our r.node.RS() to point files
+			// we have to use mapping passed by --replset-mapping option
+			rdr, err := stg.SourceReader(path.Join(bcp.Name, mapRS(r.brief.SetName), ns))
+			if err != nil {
+				return nil, err
 			}
 
-			// selective restore needs to process users and roles from the full backup,
-			// so we'll continue with selective restore
-		}
-
-		r.log.Debug("restoring up to %d collections in parallel", r.numParallelColls)
-
-		rdr, err = snapshot.DownloadDump(
-			func(ns string) (io.ReadCloser, error) {
-				stg, err := util.StorageFromConfig(&bcp.Store.StorageConf, r.log)
-				if err != nil {
-					return nil, errors.Wrap(err, "get storage")
-				}
-				// while importing backup made by RS with another name
-				// that current RS we can't use our r.node.RS() to point files
-				// we have to use mapping passed by --replset-mapping option
-				rdr, err := stg.SourceReader(path.Join(bcp.Name, mapRS(r.brief.SetName), ns))
+			if ns == archive.MetaFile {
+				data, err := io.ReadAll(rdr)
 				if err != nil {
 					return nil, err
 				}
 
-				if ns == archive.MetaFile {
-					data, err := io.ReadAll(rdr)
-					if err != nil {
-						return nil, err
-					}
-
-					err = r.loadIndexesFrom(bytes.NewReader(data))
-					if err != nil {
-						return nil, errors.Wrap(err, "load indexes")
-					}
-
-					rdr = io.NopCloser(bytes.NewReader(data))
+				err = r.loadIndexesFrom(bytes.NewReader(data), cloneNS)
+				if err != nil {
+					return nil, errors.Wrap(err, "load indexes")
 				}
 
-				return rdr, nil
-			},
-			bcp.Compression,
-			util.MakeSelectedPred(nss),
-			r.numParallelColls)
-	}
+				rdr = io.NopCloser(bytes.NewReader(data))
+			}
+
+			return rdr, nil
+		},
+		bcp.Compression,
+		util.MakeSelectedPred(nss),
+		r.numParallelColls)
 	if err != nil {
 		return err
 	}
 	defer rdr.Close()
 
-	// Restore snapshot (mongorestore)
-	err = r.snapshot(rdr)
-	if err != nil {
-		return errors.Wrap(err, "mongorestore")
+	if r.nodeInfo.IsConfigSrv() && util.IsSelective(nss) {
+		err = r.snapshot(rdr, cloneNS, true)
+		if err != nil {
+			return errors.Wrap(err, "mongorestore")
+		}
+
+		// restore cluster specific configs only
+		if err := r.configsvrRestore(ctx, bcp, nss, mapRS); err != nil {
+			return err
+		}
+	} else {
+		err = r.snapshot(rdr, cloneNS, false)
+		if err != nil {
+			return errors.Wrap(err, "mongorestore")
+		}
 	}
 
 	if usersAndRolesOpt {
@@ -853,6 +1025,86 @@ func (r *Restore) RunSnapshot(
 	return nil
 }
 
+func (r *Restore) restoreLegacyArchive(
+	ctx context.Context,
+	dump string,
+	bcp *backup.BackupMeta,
+) error {
+	sr, err := r.bcpStg.SourceReader(dump)
+	if err != nil {
+		return errors.Wrapf(err, "get object %s for the storage", dump)
+	}
+	defer sr.Close()
+
+	rdr, err := compress.Decompress(sr, bcp.Compression)
+	if err != nil {
+		return errors.Wrapf(err, "decompress object %s", dump)
+	}
+	defer rdr.Close()
+
+	// Restore snapshot (mongorestore)
+	err = r.snapshot(rdr, snapshot.CloneNS{}, false)
+	if err != nil {
+		return errors.Wrap(err, "mongorestore")
+	}
+
+	if err := r.restoreUsersAndRoles(ctx, nil); err != nil {
+		return errors.Wrap(err, "restoring users and roles")
+	}
+
+	return nil
+}
+
+func (r *Restore) checkForCompatibility(
+	ctx context.Context,
+	name string,
+	bcp *backup.BackupMeta,
+) error {
+	var rs *backup.BackupReplset
+	for i := range bcp.Replsets {
+		if bcp.Replsets[i].Name == name {
+			rs = &bcp.Replsets[i]
+			break
+		}
+	}
+	if rs == nil {
+		// it is ok if target cluster has more shards then source.
+		// however, data for configsvr is required
+		if !r.brief.ConfigSvr {
+			return ErrNoDataForConfigsvr
+		}
+
+		return ErrNoDataForShard
+	}
+
+	isRSConfigsvr := rs.IsConfigSvr != nil && *rs.IsConfigSvr
+	if isRSConfigsvr && !r.brief.ConfigSvr {
+		return errors.New("cannot restore configsvr data to non-configsvr")
+	}
+	if !isRSConfigsvr && r.brief.ConfigSvr {
+		return errors.New("cannot restore non-configsvr data to configsvr")
+	}
+
+	isConfigShard := false
+	if r.brief.ConfigSvr && r.brief.Version.IsConfigShardSupported() {
+		var err error
+		isConfigShard, err = topo.HasConfigShard(ctx, r.leadConn)
+		if err != nil {
+			return errors.Wrap(err, "has configshard")
+		}
+	}
+
+	isRSConfigShard := rs.IsConfigShard != nil && *rs.IsConfigShard
+	if isRSConfigShard && !isConfigShard {
+		return errors.New("cannot restore configshard data to non-configshard")
+	}
+	if !isRSConfigShard && isConfigShard {
+		return errors.New("cannot restore non-configshard data to configshard")
+	}
+
+	return nil
+}
+
 func (r *Restore) restoreUsersAndRoles(ctx context.Context, nss []string) error {
 	r.log.Info("restoring users and roles")
 	cusr, err := topo.CurrentUser(ctx, r.nodeConn)
@@ -873,12 +1125,15 @@ func (r *Restore) restoreUsersAndRoles(ctx context.Context, nss []string) error
 	return nil
 }
 
-func (r *Restore) loadIndexesFrom(rdr io.Reader) error {
+func (r *Restore) loadIndexesFrom(rdr io.Reader, cloneNS snapshot.CloneNS) error {
 	meta, err := archive.ReadMetadata(rdr)
 	if err != nil {
 		return errors.Wrap(err, "read metadata")
 	}
 
+	fromDB, fromColl := cloneNS.SplitFromNS()
+	toDB, toColl := cloneNS.SplitToNS()
+
 	for _, ns := range meta.Namespaces {
 		var md mongorestore.Metadata
 		err := bson.UnmarshalExtJSON([]byte(ns.Metadata), true, &md)
@@ -887,7 +1142,11 @@ func (r *Restore) loadIndexesFrom(rdr io.Reader) error {
 				ns.Database, ns.Collection)
 		}
 
-		r.indexCatalog.AddIndexes(ns.Database, ns.Collection, md.Indexes)
+		if cloneNS.IsSpecified() && ns.Database == fromDB && ns.Collection == fromColl {
+			r.indexCatalog.AddIndexes(toDB, toColl, md.Indexes)
+		} else {
+			r.indexCatalog.AddIndexes(ns.Database, ns.Collection, md.Indexes)
+		}
 
 		simple := true
 		if md.Options != nil {
@@ -1192,8 +1451,13 @@ func (r *Restore) applyOplog(ctx context.Context, ranges []oplogRange, options *
 	return nil
 }
 
-func (r *Restore) snapshot(input io.Reader) error {
-	rf, err := snapshot.NewRestore(r.brief.URI, r.cfg, r.numParallelColls)
+func (r *Restore) snapshot(input io.Reader, cloneNS snapshot.CloneNS, excludeRouterCollections bool) error {
+	rf, err := snapshot.NewRestore(
+		r.brief.URI,
+		r.cfg, cloneNS,
+		r.numParallelColls,
+		r.numInsertionWorkersPerCol,
+		excludeRouterCollections)
 	if err != nil {
 		return err
 	}
@@ -1250,7 +1514,7 @@ func (r *Restore) swapUsers(ctx context.Context, exclude *topo.AuthInfo, nss []s
 	for _, ns := range nss {
 		// ns can be "*.*" or "admin.pbmRUsers" or "admin.pbmRRoles"
 		db, _ := util.ParseNS(ns)
-		if len(db) == 0 || strings.HasPrefix(db, defs.DB) {
+		if len(db) == 0 || db == defs.DB {
 			continue
 		}
 		dbs = append(dbs, db)
