@@ -4,6 +4,7 @@ import (
 	"context"
 	"fmt"
 	"io"
+	"maps"
 	"net/http"
 	"path"
 	"runtime"
@@ -31,12 +32,83 @@ const (
 	maxBlocks = 50_000
 )
 
-type Conf struct {
-	Account     string      `bson:"account" json:"account,omitempty" yaml:"account,omitempty"`
-	Container   string      `bson:"container" json:"container,omitempty" yaml:"container,omitempty"`
-	EndpointURL string      `bson:"endpointUrl" json:"endpointUrl,omitempty" yaml:"endpointUrl,omitempty"`
-	Prefix      string      `bson:"prefix" json:"prefix,omitempty" yaml:"prefix,omitempty"`
-	Credentials Credentials `bson:"credentials" json:"-" yaml:"credentials"`
+//nolint:lll
+type Config struct {
+	Account        string            `bson:"account" json:"account,omitempty" yaml:"account,omitempty"`
+	Container      string            `bson:"container" json:"container,omitempty" yaml:"container,omitempty"`
+	EndpointURL    string            `bson:"endpointUrl" json:"endpointUrl,omitempty" yaml:"endpointUrl,omitempty"`
+	EndpointURLMap map[string]string `bson:"endpointUrlMap,omitempty" json:"endpointUrlMap,omitempty" yaml:"endpointUrlMap,omitempty"`
+	Prefix         string            `bson:"prefix" json:"prefix,omitempty" yaml:"prefix,omitempty"`
+	Credentials    Credentials       `bson:"credentials" json:"-" yaml:"credentials"`
+}
+
+func (cfg *Config) Clone() *Config {
+	if cfg == nil {
+		return nil
+	}
+
+	rv := *cfg
+	rv.EndpointURLMap = maps.Clone(cfg.EndpointURLMap)
+	return &rv
+}
+
+func (cfg *Config) Equal(other *Config) bool {
+	if cfg == nil || other == nil {
+		return cfg == other
+	}
+
+	if cfg.Account != other.Account {
+		return false
+	}
+	if cfg.Container != other.Container {
+		return false
+	}
+	if cfg.EndpointURL != other.EndpointURL {
+		return false
+	}
+	if !maps.Equal(cfg.EndpointURLMap, other.EndpointURLMap) {
+		return false
+	}
+	if cfg.Prefix != other.Prefix {
+		return false
+	}
+	if cfg.Credentials.Key != other.Credentials.Key {
+		return false
+	}
+
+	return true
+}
+
+// IsSameStorage identifies the same instance of the Azure storage.
+func (cfg *Config) IsSameStorage(other *Config) bool {
+	if cfg == nil || other == nil {
+		return cfg == other
+	}
+
+	if cfg.Account != other.Account {
+		return false
+	}
+	if cfg.Container != other.Container {
+		return false
+	}
+	if cfg.Prefix != other.Prefix {
+		return false
+	}
+	return true
+}
+
+// resolveEndpointURL returns endpoint url based on provided
+// EndpointURL or associated EndpointURLMap configuration fields.
+// If specified EndpointURLMap overrides EndpointURL field.
+func (cfg *Config) resolveEndpointURL(node string) string {
+	ep := cfg.EndpointURL
+	if epm, ok := cfg.EndpointURLMap[node]; ok {
+		ep = epm
+	}
+	if ep == "" {
+		ep = fmt.Sprintf(BlobURL, cfg.Account)
+	}
+	return ep
 }
 
 type Credentials struct {
@@ -44,18 +116,20 @@ type Credentials struct {
 }
 
 type Blob struct {
-	opts Conf
+	opts *Config
+	node string
 	log  log.LogEvent
 	// url  *url.URL
 	c *azblob.Client
 }
 
-func New(opts Conf, l log.LogEvent) (*Blob, error) {
+func New(opts *Config, node string, l log.LogEvent) (*Blob, error) {
 	if l == nil {
 		l = log.DiscardEvent
 	}
 	b := &Blob{
 		opts: opts,
+		node: node,
 		log:  l,
 	}
 
@@ -72,10 +146,17 @@ func (*Blob) Type() storage.Type {
 	return storage.Azure
 }
 
-func (b *Blob) Save(name string, data io.Reader, sizeb int64) error {
+func (b *Blob) Save(name string, data io.Reader, options ...storage.Option) error {
+	opts := storage.GetDefaultOpts()
+	for _, opt := range options {
+		if err := opt(opts); err != nil {
+			return errors.Wrap(err, "processing options for save")
+		}
+	}
+
 	bufsz := defaultUploadBuff
-	if sizeb > 0 {
-		ps := int(sizeb / maxBlocks * 11 / 10) // add 10% just in case
+	if opts.Size > 0 {
+		ps := int(opts.Size / maxBlocks * 11 / 10) // add 10% just in case
 		if ps > bufsz {
 			bufsz = ps
 		}
@@ -86,8 +167,8 @@ func (b *Blob) Save(name string, data io.Reader, sizeb int64) error {
 		cc = 1
 	}
 
-	if b.log != nil {
-		b.log.Debug("BufferSize is set to %d (~%dMb) | %d", bufsz, bufsz>>20, sizeb)
+	if b.log != nil && opts.UseLogger {
+		b.log.Debug("BufferSize is set to %d (~%dMb) | %d", bufsz, bufsz>>20, opts.Size)
 	}
 
 	_, err := b.c.UploadStream(context.TODO(),
@@ -262,10 +343,7 @@ func (b *Blob) client() (*azblob.Client, error) {
 	opts.Retry = policy.RetryOptions{
 		MaxRetries: defaultRetries,
 	}
-	epURL := b.opts.EndpointURL
-	if epURL == "" {
-		epURL = fmt.Sprintf(BlobURL, b.opts.Account)
-	}
+	epURL := b.opts.resolveEndpointURL(b.node)
 	return azblob.NewClientWithSharedKeyCredential(epURL, cred, opts)
 }
 
