@@ -36,6 +36,7 @@ var (
 
 type Client struct {
 	conn connect.Client
+	node string
 }
 
 func (c *Client) Close(ctx context.Context) error {
@@ -165,7 +166,7 @@ func (c *Client) getBackupHelper(
 	}
 
 	if options.FetchFilelist {
-		err := fillFilelistForBackup(ctx, bcp)
+		err := c.fillFilelistForBackup(ctx, bcp)
 		if err != nil {
 			return nil, errors.Wrap(err, "fetch filelist")
 		}
@@ -174,7 +175,7 @@ func (c *Client) getBackupHelper(
 	return bcp, nil
 }
 
-func fillFilelistForBackup(ctx context.Context, bcp *BackupMetadata) error {
+func (c *Client) fillFilelistForBackup(ctx context.Context, bcp *BackupMetadata) error {
 	var err error
 	var stg storage.Storage
 
@@ -182,7 +183,7 @@ func fillFilelistForBackup(ctx context.Context, bcp *BackupMetadata) error {
 	eg.SetLimit(runtime.NumCPU())
 
 	if version.HasFilelistFile(bcp.PBMVersion) {
-		stg, err = util.StorageFromConfig(&bcp.Store.StorageConf, log.LogEventFromContext(ctx))
+		stg, err = util.StorageFromConfig(&bcp.Store.StorageConf, c.node, log.LogEventFromContext(ctx))
 		if err != nil {
 			return errors.Wrap(err, "get storage")
 		}
@@ -215,7 +216,7 @@ func fillFilelistForBackup(ctx context.Context, bcp *BackupMetadata) error {
 
 			if stg == nil {
 				// in case if it is the first backup made with filelist file
-				stg, err = getStorageForRead(ctx, bcp)
+				stg, err = c.getStorageForRead(ctx, bcp)
 				if err != nil {
 					return errors.Wrap(err, "get storage")
 				}
@@ -240,8 +241,8 @@ func fillFilelistForBackup(ctx context.Context, bcp *BackupMetadata) error {
 	return eg.Wait()
 }
 
-func getStorageForRead(ctx context.Context, bcp *backup.BackupMeta) (storage.Storage, error) {
-	stg, err := util.StorageFromConfig(&bcp.Store.StorageConf, log.LogEventFromContext(ctx))
+func (c *Client) getStorageForRead(ctx context.Context, bcp *backup.BackupMeta) (storage.Storage, error) {
+	stg, err := util.StorageFromConfig(&bcp.Store.StorageConf, c.node, log.LogEventFromContext(ctx))
 	if err != nil {
 		return nil, errors.Wrap(err, "get storage")
 	}
@@ -261,8 +262,12 @@ func (c *Client) GetRestoreByOpID(ctx context.Context, opid string) (*RestoreMet
 	return restore.GetRestoreMetaByOPID(ctx, c.conn, opid)
 }
 
-func (c *Client) SyncFromStorage(ctx context.Context) (CommandID, error) {
-	opid, err := ctrl.SendResync(ctx, c.conn)
+func (c *Client) SyncFromStorage(ctx context.Context, includeRestores bool) (CommandID, error) {
+	var opts *ctrl.ResyncCmd
+	if includeRestores {
+		opts = &ctrl.ResyncCmd{IncludeRestores: true}
+	}
+	opid, err := ctrl.SendResync(ctx, c.conn, opts)
 	return CommandID(opid.String()), err
 }
 
@@ -271,12 +276,12 @@ func (c *Client) SyncFromExternalStorage(ctx context.Context, name string) (Comm
 		return NoOpID, errors.New("name is not provided")
 	}
 
-	opid, err := ctrl.SendSyncMetaFrom(ctx, c.conn, name)
+	opid, err := ctrl.SendResync(ctx, c.conn, &ctrl.ResyncCmd{Name: name})
 	return CommandID(opid.String()), err
 }
 
 func (c *Client) SyncFromAllExternalStorages(ctx context.Context) (CommandID, error) {
-	opid, err := ctrl.SendSyncMetaFrom(ctx, c.conn, "")
+	opid, err := ctrl.SendResync(ctx, c.conn, &ctrl.ResyncCmd{All: true})
 	return CommandID(opid.String()), err
 }
 
@@ -285,12 +290,12 @@ func (c *Client) ClearSyncFromExternalStorage(ctx context.Context, name string)
 		return NoOpID, errors.New("name is not provided")
 	}
 
-	opid, err := ctrl.SendClearMetaFrom(ctx, c.conn, name)
+	opid, err := ctrl.SendResync(ctx, c.conn, &ctrl.ResyncCmd{Name: name, Clear: true})
 	return CommandID(opid.String()), err
 }
 
 func (c *Client) ClearSyncFromAllExternalStorages(ctx context.Context) (CommandID, error) {
-	opid, err := ctrl.SendClearMetaFrom(ctx, c.conn, "")
+	opid, err := ctrl.SendResync(ctx, c.conn, &ctrl.ResyncCmd{All: true, Clear: true})
 	return CommandID(opid.String()), err
 }
 
